/**
 * @file pca9557_driver.c
 * @brief Driver implementation for the PCA9557 I2C I/O Expander.
 */

#include "pca9557_driver.h"
#include "../hal/twi_master.h" // Include our TWI HAL

// Conditional include for Debug UART
#ifdef DEBUG_UART_ENABLE
#include "../hal/uart_debug.h"
#endif

/**
 * @brief Initializes the PCA9557. Configures all pins as Inputs (High-Z).
 */
bool pca9557_init(void) {
    // Write 0xFF to Config Register (Reg 3) to set all pins to Input (High-Z)
    if (!twi_write_reg(PCA9557_I2C_ADDR, PCA9557_REG_CONFIG, 0xFF)) {
#ifdef DEBUG_UART_ENABLE
        uart_printf("PCA9557 Init ERROR: Failed to write Config register.\r\n");
#endif
        return false;
    }
    // Optional: Write 0x00 to Output Port Register (Reg 1) for a clean state
    if (!twi_write_reg(PCA9557_I2C_ADDR, PCA9557_REG_OUTPUT_PORT, 0x00)) {
#ifdef DEBUG_UART_ENABLE
        uart_printf("PCA9557 Init WARN: Failed to write Output Port register (continuing).\r\n");
#endif
        // Continue even if this fails, config register is more important
    }
#ifdef DEBUG_UART_ENABLE
    uart_printf("PCA9557 Initialized (All pins High-Z).\r\n");
#endif
    return true;
}

/**
 * @brief Reads a register from PCA9557. Helper function.
 */
static bool pca9557_read_register(uint8_t reg_addr, uint8_t *value) {
    return twi_read_reg(PCA9557_I2C_ADDR, reg_addr, value);
}

/**
 * @brief Configures a specific PCA9557 pin as Output HIGH.
 */
bool pca9557_set_pin_output_high(uint8_t pin) {
    if (pin > 7) return false; // Invalid pin

    uint8_t current_output_reg;
    uint8_t current_config_reg;

    // 1. Read current Output Port Register
    if (!pca9557_read_register(PCA9557_REG_OUTPUT_PORT, current_output_reg)) {
        return false; // Failed to read output reg
    }
    // 2. Set the desired pin bit to 1
    current_output_reg |= (1 << pin);
    // 3. Write the new value back to Output Port Register
    if (!twi_write_reg(PCA9557_I2C_ADDR, PCA9557_REG_OUTPUT_PORT, current_output_reg)) {
        return false; // Failed to write output reg
    }

    // 4. Read current Configuration Register
    if (!pca9557_read_register(PCA9557_REG_CONFIG, current_config_reg)) {
        // If config read fails, we might still try to set it blindly
        current_config_reg = 0xFF; // Assume all inputs initially
         #ifdef DEBUG_UART_ENABLE
         uart_printf("PCA9557 WARN: Failed read config, assuming 0xFF\r\n");
         #endif
    }
    // 5. Clear the desired pin bit to 0 (make it Output)
    current_config_reg &= ~(1 << pin);
    // 6. Write the new value back to Configuration Register
    if (!twi_write_reg(PCA9557_I2C_ADDR, PCA9557_REG_CONFIG, current_config_reg)) {
        return false; // Failed to write config reg
    }

    return true;
}

/**
 * @brief Configures a specific PCA9557 pin as Input (High-Z).
 */
bool pca9557_set_pin_input_hiz(uint8_t pin) {
    if (pin > 7) return false; // Invalid pin

    uint8_t current_config_reg;

    // 1. Read current Configuration Register
    if (!pca9557_read_register(PCA9557_REG_CONFIG, current_config_reg)) {
         // If config read fails, we might still try to set it blindly
        current_config_reg = 0x00; // Assume all outputs initially
         #ifdef DEBUG_UART_ENABLE
         uart_printf("PCA9557 WARN: Failed read config, assuming 0x00\r\n");
         #endif
    }
    // 2. Set the desired pin bit to 1 (make it Input / High-Z)
    current_config_reg |= (1 << pin);
    // 3. Write the new value back to Configuration Register
    if (!twi_write_reg(PCA9557_I2C_ADDR, PCA9557_REG_CONFIG, current_config_reg)) {
        return false; // Failed to write config reg
    }

    // Optional: Clear the corresponding bit in the output register
    uint8_t current_output_reg;
    if (pca9557_read_register(PCA9557_REG_OUTPUT_PORT, current_output_reg)) {
        current_output_reg &= ~(1 << pin);
        twi_write_reg(PCA9557_I2C_ADDR, PCA9557_REG_OUTPUT_PORT, current_output_reg);
        // Ignore write error here, config is more important
    }


    return true;
}

/**
 * @brief Reads the state of the input port register.
 */
bool pca9557_read_input_register(uint8_t *input_value) {
    return pca9557_read_register(PCA9557_REG_INPUT_PORT, input_value);
}
