/**
 * @file pca9557_driver.h
 * @brief Driver for the PCA9557 I2C I/O Expander.
 */

#ifndef PCA9557_DRIVER_H
#define PCA9557_DRIVER_H

#include <stdint.h>
#include <stdbool.h>

#define PCA9557_I2C_ADDR 0x18 // Default address with A0-A2 grounded

// Register addresses for PCA9557
#define PCA9557_REG_INPUT_PORT  0x00
#define PCA9557_REG_OUTPUT_PORT 0x01
#define PCA9557_REG_POL_INV     0x02 // Polarity Inversion
#define PCA9557_REG_CONFIG      0x03 // Configuration (Direction: 0=Output, 1=Input)

/**
 * @brief Initializes the PCA9557.
 * Configures all pins as Inputs (High-Z) by default.
 *
 * @return true on success, false on I2C communication error.
 */
bool pca9557_init(void);

/**
 * @brief Configures a specific PCA9557 pin as Output HIGH.
 * Used to enable a sensor group VCC.
 *
 * @param pin The pin number (0-7) corresponding to the sensor group VCC line.
 * @return true on success, false on I2C communication error.
 */
bool pca9557_set_pin_output_high(uint8_t pin);

/**
 * @brief Configures a specific PCA9557 pin as Input (High-Z).
 * Used to disable a sensor group VCC without pulling it LOW.
 *
 * @param pin The pin number (0-7) corresponding to the sensor group VCC line.
 * @return true on success, false on I2C communication error.
 */
bool pca9557_set_pin_input_hiz(uint8_t pin);

/**
 * @brief Reads the state of the input port register.
 * Note: Only pins configured as inputs will reflect the external state.
 *
 * @param input_value Pointer to store the read byte.
 * @return true on success, false on I2C communication error.
 */
bool pca9557_read_input_register(uint8_t *input_value);

#endif // PCA9557_DRIVER_H
