/**
 * @file crc8.h
 * @brief Provides CRC8 calculation using polynomial 0x07 (CRC-8-CCITT/ATM).
 *
 * This file contains the optimized inline assembly function from avr-libc's
 * util/crc16.h for calculating CRC8 with the polynomial 0x07 (x^8 + x^2 + x + 1),
 * which is required by the TMAG5273 sensor.
 *
 * A helper function `calculate_tmag_crc8` is provided for easier use with data buffers.
 */

#ifndef CRC8_H
#define CRC8_H

#include <stdint.h>

// --- Core CRC8 Update Function (Polynomial 0x07) ---

/** \ingroup util_crc
    @brief Optimized CRC-8-CCITT calculation (Polynomial 0x07).

    Polynomial: x^8 + x^2 + x + 1 (0x07, reflected 0xE0)<br>

    This function updates the CRC value for a single byte of data.

    For use with simple CRC-8<br>
    Initial value: 0x0

    For use with CRC-8-ROHC<br>
    Initial value: 0xff<br>
    Reference: http://tools.ietf.org/html/rfc3095#section-5.9.1

    For use with CRC-8-ATM/ITU (and **TMAG5273**)<br>
    Initial value: **0xff**<br>
    Final XOR value: 0x00 (for TMAG5273, some protocols use 0x55)<br>
    Reference: http://www.itu.int/rec/T-REC-I.432.1-199902-I/en

    The C equivalent has been originally written by Dave Hylands.
    Assembly code is based on _crc_ibutton_update optimization.

    \code
    // C Equivalent Logic:
    uint8_t _crc8_ccitt_update (uint8_t inCrc, uint8_t inData)
    {
        uint8_t   i;
        uint8_t   data;

        data = inCrc ^ inData;

        for ( i = 0; i < 8; i++ )
        {
            // Check if MSB is set
            if (( data & 0x80 ) != 0 )
            {
                data <<= 1;
                data ^= 0x07; // Polynomial 0x07
            }
            else
            {
                data <<= 1;
            }
        }
        return data;
    }
    \endcode

    @param __crc The previous CRC8 value.
    @param __data The byte of data to include in the CRC calculation.
    @return The updated CRC8 value.
*/
static __inline__ uint8_t
_crc8_poly07_update(uint8_t __crc, uint8_t __data) // Renamed slightly for clarity
{
    // This is the _crc8_ccitt_update function from <util/crc16.h>
    // It uses polynomial 0x07.
    uint8_t __i, __pattern;
    __asm__ __volatile__ (
        "    eor    %0, %4" "\n\t"  // crc ^= data
        "    ldi    %1, 8" "\n\t"   // i = 8
        "    ldi    %2, 0x07" "\n\t"// pattern = 0x07 (Polynomial)
        "1:  lsl    %0" "\n\t"      // crc <<= 1
        "    brcc   2f" "\n\t"      // if carry is clear (msb was 0), skip xor
        "    eor    %0, %2" "\n\t"  // crc ^= pattern
        "2:  dec    %1" "\n\t"      // i--
        "    brne   1b" "\n\t"      // loop if i != 0
        : "=r" (__crc), "=d" (__i), "=d" (__pattern) // Output operands
        : "0" (__crc), "r" (__data)                // Input operands
        : /* No clobbers */ );
    return __crc;
}

// --- Helper / Wrapper Function ---

/**
 * @brief Calculates the CRC8 (Polynomial 0x07, Initial Value 0xFF) for a data buffer.
 *
 * This is specifically tailored for the TMAG5273 sensor communication requirements.
 *
 * @param data Pointer to the data buffer.
 * @param length Number of bytes in the data buffer.
 * @return The calculated 8-bit CRC value.
 */
static inline uint8_t calculate_tmag_crc8(const uint8_t *data, uint8_t length) {
    uint8_t crc = 0xFF; // Initial value required by TMAG5273
    for (uint8_t i = 0; i < length; i++) {
        crc = _crc8_poly07_update(crc, data[i]); // Use the inline function
    }
    // No final XOR required for TMAG5273
    return crc;
}

#endif // CRC8_H
