/**
 * @file timer.c
 * @brief Implementation of non-blocking timer functions using TCB0.
 */

#include "timer.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/atomic.h>

// Global variable holding the system tick count in milliseconds.
// Declared volatile because it's modified in an ISR and accessed elsewhere.
static volatile uint32_t system_ticks_ms = 0;

/**
 * @brief Initializes Timer/Counter B0 (TCB0) for a 1ms system tick.
 * Assumes F_CPU = 16000000UL.
 */
void timer_init(void) {
    // --- Configure TCB0 for 1ms periodic interrupt ---

#if F_CPU != 16000000UL
#warning "timer_init() assumes F_CPU is 16MHz for 1ms tick calculation!"
#endif

    // 1. Select Clock Source and Prescaler
    // CLK_PER = 16 MHz. Use CLK_PER / 2 = 8 MHz for TCB.
    TCB0.CTRLA = TCB_CLKSEL_DIV2_gc; // Korrigierter Name gemäß Compiler-Vorschlag/Header

    // 2. Set Timer Mode to Periodic Interrupt
    TCB0.CTRLB = TCB_CNTMODE_INT_gc;

    // 3. Set Compare/Capture Value for 1ms period
    // Target cycles = 0.001 s * 8,000,000 Hz = 8000 cycles
    // Set CCMP to N-1 for N cycles period in this mode.
    TCB0.CCMP = 8000 - 1; // 7999

    // 4. Enable Capture/Compare Interrupt
    TCB0.INTCTRL = TCB_CAPT_bm;

    // 5. Enable the Timer
    // Note: Global interrupts (sei()) must be called elsewhere for the ISR to run.
    TCB0.CTRLA |= TCB_ENABLE_bm;
}

/**
 * @brief TCB0 Interrupt Service Routine (ISR).
 * Called every 1ms (based on timer_init configuration).
 * Increments the system tick counter.
 */
ISR(TCB0_INT_vect) {
    // Increment the millisecond counter
    system_ticks_ms++;

    // Clear the interrupt flag (important!)
    // The flag is cleared by writing a '1' to it.
    TCB0.INTFLAGS = TCB_CAPT_bm;
}

/**
 * @brief Gets the current system time in milliseconds. Atomically reads the tick counter.
 */
uint32_t timer_get_ms(void) {
    uint32_t ticks;
    // Ensure atomic read of the 32-bit variable, as the ISR could modify it
    // during the read operation on an 8-bit architecture.
    ATOMIC_BLOCK(ATOMIC_RESTORESTATE) {
        ticks = system_ticks_ms;
    }
    return ticks;
}

/* Example usage of non-blocking delay:
void some_function() {
    static uint32_t last_action_time = 0;
    uint32_t current_time = timer_get_ms();

    // Perform action every 500ms
    if ((current_time - last_action_time) >= 500) {
        last_action_time = current_time; // Update last action time

        // --- Perform the periodic action here ---
        // e.g., toggle an LED, send a status message
    }

    // --- Other non-blocking tasks can run here continuously ---
}
*/
