/**
 * @file timer.h
 * @brief Non-blocking timer functions using Timer/Counter B0 (TCB0) for a system tick.
 */

#ifndef TIMER_H
#define TIMER_H

#include <stdint.h>

/**
 * @brief Initializes Timer/Counter B0 (TCB0) to generate a periodic interrupt.
 *
 * Configures TCB0 to generate an interrupt typically every 1ms,
 * incrementing a global system tick counter.
 * This requires F_CPU to be correctly defined (16000000UL expected).
 * Global interrupts must be enabled (`sei()`) after calling this function
 * for the timer to work.
 */
void timer_init(void);

/**
 * @brief Gets the current system time in milliseconds since timer initialization.
 *
 * Reads the global system tick counter atomically.
 *
 * @return uint32_t The number of milliseconds elapsed.
 */
uint32_t timer_get_ms(void);

/**
 * @brief Provides a non-blocking delay for the specified number of milliseconds.
 *
 * This function is intended to be used within loops where other tasks can be
 * processed while waiting. It checks if the specified duration has passed
 * since the provided start time.
 *
 * @param start_time_ms The time when the delay period started (obtained via timer_get_ms()).
 * @param duration_ms The duration of the delay in milliseconds.
 * @return true if the delay duration has passed, false otherwise.
 */
// Example helper - can be expanded or used directly in loops
// bool timer_delay_expired(uint32_t start_time_ms, uint32_t duration_ms);
// --> Besser direkt die Logik in der Schleife verwenden, siehe Beispiel in timer.c

#endif // TIMER_H
