/**
 * @file twi_master.c
 * @brief TWI (I2C) Master driver implementation (Blocking) for ATtiny826.
 */

#include "twi_master.h"
#include <avr/io.h>
#include <util/delay.h> // Only used for very short delay in start recovery

// Conditional include for Debug UART
#ifdef DEBUG_UART_ENABLE
#include "uart_debug.h"
#endif

#if F_CPU != 16000000UL
#warning "TWI Master code assumes F_CPU = 16MHz for baud calculations!"
#endif

/**
 * @brief Waits for specific TWI flags or errors with a software timeout.
 * @param flags_to_wait_for Bitmask of flags (like TWI_WIF_bm, TWI_RIF_bm) to wait for.
 * @return true if one of the expected flags is set without error/timeout, false otherwise.
 */
static bool twi_wait(uint8_t flags_to_wait_for) {
    uint16_t timeout_count = TWI_DEFAULT_TIMEOUT;
    // Wait while NONE of the wait_flags are set AND no error flags are set AND timeout > 0
    while ( !(TWI0.MSTATUS & (flags_to_wait_for | TWI_ARBLOST_bm | TWI_BUSERR_bm)) && (--timeout_count > 0) );

    if (timeout_count == 0) {
#ifdef DEBUG_UART_ENABLE
        uart_printf("TWI TIMEOUT! MSTATUS=0x%02X\r\n", TWI0.MSTATUS);
#endif
        // Try to recover by forcing Idle state (might not always work)
        TWI0.MSTATUS |= TWI_BUSSTATE_IDLE_gc;
        return false;
    }
    if (TWI0.MSTATUS & (TWI_ARBLOST_bm | TWI_BUSERR_bm)) {
#ifdef DEBUG_UART_ENABLE
        uart_printf("TWI ERROR! Flags: %s %s MSTATUS=0x%02X\r\n",
                    (TWI0.MSTATUS & TWI_ARBLOST_bm) ? "ARBLOST" : "",
                    (TWI0.MSTATUS & TWI_BUSERR_bm) ? "BUSERR" : "",
                    TWI0.MSTATUS);
#endif
        // Try to recover
        TWI0.MSTATUS |= TWI_BUSSTATE_IDLE_gc;
        return false;
    }
    // If we are here, one of the flags_to_wait_for must be set without error
    return true;
}

/**
 * @brief Initializes the TWI0 peripheral as Master.
 */
void twi_init(uint32_t twi_freq) {
    // --- Pin Configuration ---
    // External Pull-ups are assumed and required. Disable internal ones.
    PORTB.PIN0CTRL &= ~PORT_PULLUPEN_bm; // SCL (PB0)
    PORTB.PIN1CTRL &= ~PORT_PULLUPEN_bm; // SDA (PB1)

    // --- Frequency Setup (using formula for 16MHz F_CPU) ---
    uint8_t baud_value = 75; // Default to 100kHz @ 16MHz
    if (twi_freq >= 400000) {
        baud_value = 15; // 400kHz @ 16MHz
    } // Add more calculations if needed for other frequencies

    TWI0.MBAUD = baud_value;

    // --- Enable TWI Master ---
    // Using default disabled timeout for now.
    TWI0.MCTRLA = TWI_ENABLE_bm;
                  // | TWI_TIMEOUT_DISABLED_gc; // Or specific timeout

    // --- Force Bus State to Idle ---
    // This is crucial after init or error recovery.
    TWI0.MSTATUS = TWI_BUSSTATE_IDLE_gc; // Write 1 to BUSSTATE forces IDLE
}

/**
 * @brief Sends a START condition and slave address (blocking).
 */
bool twi_start(uint8_t slave_addr, bool write_mode) {
    // 1. Check if bus is idle (optional, but good practice)
    if ((TWI0.MSTATUS & TWI_BUSSTATE_gm) != TWI_BUSSTATE_IDLE_gc) {
        #ifdef DEBUG_UART_ENABLE
        uart_printf("TWI Start: Bus not idle (0x%02X), forcing idle.\r\n", (TWI0.MSTATUS & TWI_BUSSTATE_gm));
        #endif
        TWI0.MSTATUS = TWI_BUSSTATE_IDLE_gc; // Force idle
        // Wait briefly after forcing idle - use blocking delay here exceptionally for simplicity
        _delay_us(5);
         if ((TWI0.MSTATUS & TWI_BUSSTATE_gm) != TWI_BUSSTATE_IDLE_gc) {
             #ifdef DEBUG_UART_ENABLE
             uart_printf("TWI Start: Failed to force bus idle.\r\n");
             #endif
             return false;
         }
    }

    // 2. Send Address + R/W bit (This implicitly generates START)
    uint8_t addr_rw = (slave_addr << 1) | (write_mode ? 0 : 1);
    TWI0.MADDR = addr_rw;

    // 3. Wait for Write/Read Interrupt Flag (WIF/RIF) or Error/Timeout
    uint8_t wait_flags = write_mode ? TWI_WIF_bm : TWI_RIF_bm;
    if (!twi_wait(wait_flags)) {
        // twi_wait handles error reporting and attempts recovery
        return false;
    }

    // 4. Check for NACK (RXACK bit is 1 if NACK received)
    if (TWI0.MSTATUS & TWI_RXACK_bm) {
        #ifdef DEBUG_UART_ENABLE
        // Don't print for every NACK during scan, handled in twi_scan_bus
        // uart_printf("TWI Start: NACK received for addr 0x%02X\r\n", slave_addr);
        #endif
        TWI0.MCTRLB = TWI_MCMD_STOP_gc; // Send STOP condition after NACK
        return false;
    }

    // If we reach here, START was successful and ACK was received.
    return true;
}

/**
 * @brief Sends a STOP condition (blocking).
 */
void twi_stop(void) {
    // Send the STOP command
    TWI0.MCTRLB = TWI_MCMD_STOP_gc;

    // Optional: Wait briefly for bus to actually become idle.
    // Use a simple loop here as a timeout isn't critical like during data transfer.
    //volatile uint16_t timeout = 500;
    //while (((TWI0.MSTATUS & TWI_BUSSTATE_gm) != TWI_BUSSTATE_IDLE_gc) && (--timeout > 0));
    // A short fixed delay might be sufficient and simpler:
     _delay_us(5); // Give time for STOP to propagate
}


/**
 * @brief Scans the I2C bus for responding devices (blocking).
 */
uint8_t twi_scan_bus(void) {
#ifdef DEBUG_UART_ENABLE
    uart_printf("Scanning I2C Bus (0x08-0x77)...\r\n");
#endif
    uint8_t devices_found = 0;

    for (uint8_t addr = 0x08; addr <= 0x77; addr++) {
        // Attempt to start communication with the address in WRITE mode
        if (twi_start(addr, true)) {
            // If start is successful (ACK received): Device found!
#ifdef DEBUG_UART_ENABLE
            uart_printf(" Device found at 0x%02X !\r\n", addr);
#endif
            devices_found++;
            // Send STOP condition immediately after finding the device
            twi_stop();
        } else {
            // twi_start failed (NACK, timeout, or bus error).
            // twi_start already sent STOP on NACK/error.
            // Ensure bus is idle before trying next address.
             uint16_t timeout_idle = 5000;
             while (((TWI0.MSTATUS & TWI_BUSSTATE_gm) != TWI_BUSSTATE_IDLE_gc) && (--timeout_idle > 0));
             if(timeout_idle == 0) {
                 #ifdef DEBUG_UART_ENABLE
                 uart_printf("WARN: Bus not idle after failed start for 0x%02X\r\n", addr);
                 #endif
                 // Attempt to force idle again before proceeding
                 TWI0.MSTATUS = TWI_BUSSTATE_IDLE_gc;
                  _delay_us(5);
             }
        }
         // Optional: Short delay between trying addresses
         // _delay_us(10);
    }

#ifdef DEBUG_UART_ENABLE
    uart_printf("I2C Scan finished. Found %u device(s).\r\n", devices_found);
#endif
    return devices_found;
}


/**
 * @brief Writes a single data byte after START and address phase (blocking).
 *        Helper for twi_write_reg.
 * @param data Data byte to write.
 * @return true on success (ACK received), false on NACK/error/timeout.
 */
static bool twi_write_data(uint8_t data) {
    // Wait until ready to transmit data (WIF should be set from address phase or previous write)
    if (!twi_wait(TWI_WIF_bm)) {
        return false; // Timeout or bus error before sending data
    }

    // Send data byte
    TWI0.MDATA = data;

    // Wait for transmission complete (WIF set again) or error
    if (!twi_wait(TWI_WIF_bm)) {
        return false; // Timeout or bus error after sending data
    }

    // Check for NACK
    if (TWI0.MSTATUS & TWI_RXACK_bm) {
#ifdef DEBUG_UART_ENABLE
        uart_printf("TWI Write: NACK received after data byte 0x%02X\r\n", data);
#endif
        TWI0.MCTRLB = TWI_MCMD_STOP_gc; // Send STOP condition after NACK
        return false;
    }

    return true; // Write successful
}

/**
 * @brief Reads a single data byte with ACK/NACK control (blocking).
 *        Helper for twi_read_reg.
 * @param send_ack true to send ACK (more bytes to read), false for NACK (last byte).
 * @param read_data Pointer to store the read byte.
 * @return true on success, false on error/timeout.
 */
static bool twi_read_data(bool send_ack, uint8_t *read_data) {
     // Wait until data is received (RIF should be set from address phase or previous read)
    if (!twi_wait(TWI_RIF_bm)) {
        TWI0.MCTRLB = TWI_MCMD_STOP_gc; // Ensure stop on error/timeout
        return false;
    }

    // Read data byte
    *read_data = TWI0.MDATA;

    // Send ACK or NACK command
    if (send_ack) {
        TWI0.MCTRLB = TWI_MCMD_RECVTRANS_gc; // Send ACK, expect more data
    } else {
        TWI0.MCTRLB = TWI_ACKACT_NACK_gc | TWI_MCMD_RECVTRANS_gc; // Send NACK, this was last byte
    }

    return true;
}


/**
 * @brief Writes a single byte to a specific register of an I2C slave (blocking).
 */
bool twi_write_reg(uint8_t dev_addr, uint8_t reg_addr, uint8_t data) {
    if (!twi_start(dev_addr, true)) { // Start + Slave Address + Write
        return false;
    }
    if (!twi_write_data(reg_addr)) { // Write Register Address
        // twi_write_data sends STOP on NACK/Error
        return false;
    }
    if (!twi_write_data(data)) { // Write Data Byte
        // twi_write_data sends STOP on NACK/Error
        return false;
    }
    twi_stop(); // Send STOP
    return true;
}


/**
 * @brief Reads a single byte from a specific register of an I2C slave (blocking).
 */
bool twi_read_reg(uint8_t dev_addr, uint8_t reg_addr, uint8_t *read_data) {
    // 1. Write the register address we want to read from
    if (!twi_start(dev_addr, true)) { // Start + Slave Address + Write
        return false;
    }
    if (!twi_write_data(reg_addr)) { // Write Register Address
        // twi_write_data sends STOP on NACK/Error
        return false;
    }
    // Send STOP after writing register address (Standard Vorgehen vieler Sensoren)
    // twi_stop(); // Alternativ: Repeated Start senden
    // _delay_us(2); // Kurze Pause nach Stop kann helfen

    // 2. Read the data from that register
    if (!twi_start(dev_addr, false)) { // (Repeated) Start + Slave Address + Read
         // If start fails here, bus might be stuck, try STOP
         twi_stop();
        return false;
    }
    // Read single byte, send NACK (since it's the last/only byte), store in *read_data
    if (!twi_read_data(false, read_data)) {
        // twi_read_data sends STOP on error
        return false;
    }
    twi_stop(); // Send STOP

    return true;
}


// --- Placeholder für future byte read/write functions ---

// --- Implement other TWI functions (twi_write, twi_read etc.) later ---
