/**
 * @file twi_master.h
 * @brief TWI (I2C) Master driver for ATtiny826 (Blocking Implementation based on example).
 *
 * Provides basic TWI Master initialization and bus scanning functionality.
 * Uses blocking waits with timeouts for simplicity in initial testing.
 */

#ifndef TWI_MASTER_H
#define TWI_MASTER_H

#include <stdint.h>
#include <stdbool.h>

// Define a default timeout for TWI operations (number of polling loops)
// Adjust based on clock speed and required bus response time.
#define TWI_DEFAULT_TIMEOUT 10000

/**
 * @brief Initializes the TWI0 peripheral as Master.
 *
 * Configures necessary pins (PB0/SCL, PB1/SDA - internal pull-ups OFF).
 * Sets the TWI frequency based on F_CPU (expects 16MHz).
 * Enables the TWI Master interface and forces bus state to Idle.
 *
 * @param twi_freq Target TWI frequency in Hz (e.g., 100000 for 100kHz).
 */
void twi_init(uint32_t twi_freq);

/**
 * @brief Scans the I2C bus for responding devices (blocking).
 *
 * Iterates through standard 7-bit addresses (0x08 to 0x77) and attempts
 * to get an ACK from each address using twi_start() and twi_stop().
 * Uses internal blocking waits with timeouts.
 * Found device addresses are printed via UART if DEBUG_UART_ENABLE is defined.
 *
 * @return uint8_t The number of devices found.
 */
uint8_t twi_scan_bus(void);

/**
 * @brief Sends a START condition and slave address (blocking).
 *
 * Waits for the bus to be idle, then sends START + Address + R/W bit.
 * Waits for ACK/NACK or error/timeout. Sends STOP on NACK/Error.
 *
 * @param slave_addr 7-bit slave address.
 * @param write_mode true for Write operation, false for Read operation.
 * @return true if start was successful and ACK received, false otherwise.
 */
bool twi_start(uint8_t slave_addr, bool write_mode);

/**
 * @brief Sends a STOP condition (blocking).
 *
 * Sends the STOP command and waits briefly for the bus to become idle.
 */
void twi_stop(void);

/**
 * @brief Writes a single byte to a specific register of an I2C slave (blocking).
 *
 * Performs: START -> SlaveAddr+W -> ACK -> RegAddr -> ACK -> Data -> ACK -> STOP
 *
 * @param dev_addr 7-bit slave address.
 * @param reg_addr Register address to write to.
 * @param data Data byte to write.
 * @return true on success (all ACKs received), false on any error/NACK/timeout.
 */
bool twi_write_reg(uint8_t dev_addr, uint8_t reg_addr, uint8_t data);

/**
 * @brief Reads a single byte from a specific register of an I2C slave (blocking).
 *
 * Performs: START -> SlaveAddr+W -> ACK -> RegAddr -> ACK -> ReSTART -> SlaveAddr+R -> ACK -> Data -> NACK -> STOP
 *
 * @param dev_addr 7-bit slave address.
 * @param reg_addr Register address to read from.
 * @param read_data Pointer to store the read byte.
 * @return true on success, false on any error/NACK/timeout.
 */
bool twi_read_reg(uint8_t dev_addr, uint8_t reg_addr, uint8_t *read_data);

// --- Placeholder für zukünftige Funktionen (Write/Read Bytes) ---
// bool twi_write_bytes(uint8_t dev_addr, uint8_t reg_addr, const uint8_t *buffer, uint8_t length);
// bool twi_read_bytes(uint8_t dev_addr, uint8_t reg_addr, uint8_t *buffer, uint8_t length);

// --- Placeholder for future functions (Write/Read) ---
// bool twi_write(uint8_t data);
// uint8_t twi_read(bool send_ack);
// bool twi_read_bytes(uint8_t dev_addr, uint8_t reg_addr, uint8_t *buffer, uint8_t length);
// bool twi_write_bytes(uint8_t dev_addr, uint8_t reg_addr, const uint8_t *buffer, uint8_t length);


#endif // TWI_MASTER_H
