#include "uart_debug.h"
#include <avr/io.h>
#include <util/delay.h>
#include <stdarg.h>
#include <stdio.h>

// Puffer für sprintf
static char printf_buffer[64];

void uart_init(uint32_t baud_rate) {
    // Berechne BAUD Wert basierend auf F_CPU und baud_rate (Asynchron, Normal Speed S=16)
    // Formel: BAUD = (64 * F_CPU) / (S * BAUD_RATE) = (4 * F_CPU) / BAUD_RATE
    uint16_t baud_setting = (uint16_t)((((uint64_t)F_CPU * 4) / baud_rate));

    // Setze BAUD Register für USART0
    USART0.BAUD = baud_setting;

    // Konfiguriere UART: 8 Datenbits, keine Parität, 1 Stopbit
    USART0.CTRLC = USART_CHSIZE_8BIT_gc;
    
    // Aktiviere Sender und Empfänger
    USART0.CTRLB = USART_TXEN_bm | USART_RXEN_bm;
    
    // Konfiguriere Pins
    // PA1 als TXD (Ausgang)
    PORTA.DIRSET = PIN1_bm;
    // PA2 als RXD (Eingang)
    PORTA.DIRCLR = PIN2_bm;
    // Pull-up für RXD nicht aktivieren, da der Pegel vom TXB0104 aktiv getrieben wird.
    // PORTA.PIN2CTRL = PORT_PULLUPEN_bm; // Entfernt gemäß TXB0104 Datenblatt Empfehlung

    // Alternative Pin-Konfiguration für USART0 (PA1=TX, PA2=RX)
    PORTMUX.USARTROUTEA = PORTMUX_USART0_ALT1_gc;

    // Kurze Wartezeit nach Initialisierung
    _delay_ms(10);
}

void uart_send_byte(uint8_t data) {
    // Warte bis Sendepuffer bereit ist
    while (!(USART0.STATUS & USART_DREIF_bm));
    
    // Sende Daten
    USART0.TXDATAL = data;
}

void uart_send_string(const char* str) {
    while (*str) {
        uart_send_byte(*str++);
    }
}

void uart_printf(const char* format, ...) {
    va_list args;
    va_start(args, format);
    
    // Formatiere den String in den Puffer
    vsnprintf(printf_buffer, sizeof(printf_buffer), format, args);
    
    // Sende den formatierten String
    uart_send_string(printf_buffer);
    
    va_end(args);
}

bool uart_data_available(void) {
    return (USART0.STATUS & USART_RXCIF_bm);
}

uint8_t uart_read_byte(void) {
    // Warte auf Daten
    while (!uart_data_available());
    
    // Lese und gib Daten zurück
    return USART0.RXDATAL;
}
