/**
 * @file main.c
 * @brief Main application file for the Magnetic Position Sensor.
 *
 * Handles initialization and the main application loop.
 * Includes conditional code for UART debugging via DEBUG_UART_ENABLE flag,
 * including a sequential I2C scan test for sensor groups.
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/cpufunc.h> // For _PROTECTED_WRITE
#include <stdbool.h>
#include <util/delay.h> // Only for initial startup delay

// Include HAL modules
#include "hal/timer.h"
#include "hal/gpio.h"
#include "hal/power.h"
#include "hal/twi_master.h"

// Include drivers
#include "drivers/pca9557_driver.h"

// Conditional include for Debug UART
#ifdef DEBUG_UART_ENABLE
#include "hal/uart_debug.h"
#include <stdio.h>
#endif

// --- Defines ---
#define SENSOR_GROUP_COUNT 6
// PCA9557 Pin Mapping according to Spec V1.4 Table 10.6
const uint8_t pca_pin_for_group[SENSOR_GROUP_COUNT] = {
    1, // Group 1 -> IO1
    6, // Group 2 -> IO6
    5, // Group 3 -> IO5
    7, // Group 4 -> IO7
    4, // Group 5 -> IO4
    3  // Group 6 -> IO3
};


// --- Function Prototypes ---
void initialize_system(void);
void main_loop_tasks(void);
#ifdef DEBUG_UART_ENABLE
void test_sensor_groups_sequentially(void);
#endif

// --- Initialization ---
void initialize_system(void) {
    // 1. Clock
    _PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0x00); // 16MHz

    // 2. GPIOs (Placeholder - move to gpio.c)
    PORTC.DIRSET = PIN3_bm | PIN1_bm | PIN2_bm; // OE, LED_R, LED_G
    PORTC.OUTSET = PIN3_bm; // OE High
    PORTC.OUTCLR = PIN1_bm | PIN2_bm; // LEDs Off
    PORTA.DIRCLR = PIN6_bm | PIN7_bm; PORTA.PIN6CTRL = PORT_PULLUPEN_bm; PORTA.PIN7CTRL = PORT_PULLUPEN_bm;
    PORTB.DIRCLR = PIN7_bm; PORTB.PIN7CTRL = PORT_PULLUPEN_bm;
    // Pin config for UART/SPI done below

    // 3. Timer
    timer_init();

    // 4. UART / SPI Init based on build flags
#ifdef DEBUG_UART_ENABLE
    PORTA.DIRSET = PIN1_bm; // TX Output
    PORTA.DIRCLR = PIN2_bm; // RX Input
    uart_init(9600);
#else
    PORTA.DIRCLR = PIN1_bm | PIN3_bm | PIN4_bm; // MOSI, SCK, SS Inputs
    PORTA.DIRSET = PIN2_bm; // MISO Output
    // spi_slave_init();
#endif

    // 5. TWI Master Init
    twi_init(100000); // 100kHz

    // 6. PCA9557 Init
    if (!pca9557_init()) { // Sets all pins to High-Z
#ifdef DEBUG_UART_ENABLE
        uart_printf("FATAL: PCA9557 Init Failed!\r\n");
#endif
        while(1) { /* Halt */ }
    }

    // 7. Other Inits...

    // 8. Enable Global Interrupts
    sei();

      
#ifdef DEBUG_UART_ENABLE
    uart_printf("\r\n--- System Initialized (DEBUG MODE) ---\r\n");
    // Zeige den tatsächlichen MBAUD Wert, nicht den von 100kHz hartcodiert
    uart_printf("F_CPU: %lu Hz, TWI Freq Set: 100kHz, TWI0.MBAUD = %u\r\n", F_CPU, TWI0.MBAUD);
#endif

}

// ... Includes (inkl. timer.h, twi_master.h, pca9557_driver.h, uart_debug.h) ...

#ifdef DEBUG_UART_ENABLE

// --- Test State Machine ---
typedef enum {
    TEST_INIT,
    TEST_SET_HIZ_START,
    TEST_SET_HIZ_WAIT,
    TEST_UART_CHECK_A_START,
    TEST_UART_CHECK_A_WAIT,
    TEST_SET_HIGH_START,
    TEST_SET_HIGH_WAIT_I2C,
    TEST_SET_HIGH_WAIT_SETTLE,
    TEST_UART_CHECK_B_START,
    TEST_UART_CHECK_B_WAIT,
    TEST_I2C_CHECK_1_START,
    TEST_I2C_CHECK_1_WAIT,
    TEST_RESET_HIZ_START,
    TEST_RESET_HIZ_WAIT_I2C,
    TEST_RESET_HIZ_WAIT_SETTLE,
    TEST_UART_CHECK_C_START,
    TEST_UART_CHECK_C_WAIT,
    TEST_I2C_CHECK_2_START,
    TEST_I2C_CHECK_2_WAIT,
    TEST_FINISHED
} test_state_t;

static test_state_t current_test_state = TEST_INIT;
static uint32_t state_enter_time = 0;
static uint8_t test_pin = 0; // Wird in INIT gesetzt
static uint8_t read_value_buffer = 0; // Für I2C Reads
static bool i2c_op_success = false;

// --- Test-Funktion (jetzt als State Machine) ---
void run_pca_output_switch_test_step(void) {
    uint32_t current_time = timer_get_ms();
    uint32_t time_in_state = current_time - state_enter_time;

    switch (current_test_state) {
        case TEST_INIT:
            uart_printf("\r\n--- Testing PCA9557 Output Switching (State Machine) ---\r\n");
            test_pin = pca_pin_for_group[4]; // Test mit Pin für Gruppe 1 (IO1)
            state_enter_time = current_time;
            current_test_state = TEST_SET_HIZ_START;
            break;

        case TEST_SET_HIZ_START:
            uart_printf("Setting Pin %u to Input (High-Z)...\r\n", test_pin);
            i2c_op_success = pca9557_set_pin_input_hiz(test_pin);
            uart_printf("Result: %s\r\n", i2c_op_success ? "OK" : "FAIL");
            state_enter_time = current_time;
            current_test_state = TEST_SET_HIZ_WAIT;
            break;

        case TEST_SET_HIZ_WAIT:
            if (time_in_state >= 500) { // Warte 500ms
                state_enter_time = current_time;
                current_test_state = TEST_UART_CHECK_A_START;
            }
            break;

        case TEST_UART_CHECK_A_START:
            uart_printf("UART Check BEFORE switch: Sending 'A'\r\n");
            uart_send_byte('A');
            state_enter_time = current_time;
            current_test_state = TEST_UART_CHECK_A_WAIT;
            break;

        case TEST_UART_CHECK_A_WAIT:
             if (time_in_state >= 50) { // Warte 50ms
                state_enter_time = current_time;
                current_test_state = TEST_SET_HIGH_START;
            }
            break;

        case TEST_SET_HIGH_START:
            uart_printf("Setting Pin %u to Output HIGH...\r\n", test_pin);
            // --- Kritische Operation ---
            i2c_op_success = pca9557_set_pin_output_high(test_pin);
            uart_printf("Result: %s\r\n", i2c_op_success ? "OK" : "FAIL");
            // -------------------------
            state_enter_time = current_time;
            current_test_state = TEST_SET_HIGH_WAIT_I2C; // Direkt zum nächsten Zustand
            break;

         case TEST_SET_HIGH_WAIT_I2C:
             // Warte bis I2C sicher abgeschlossen ist (unsere blockierende Funktion tut das schon)
             // Füge hier eine kurze *zusätzliche* non-blocking Wartezeit ein
             if (time_in_state >= 10) { // Warte 10ms NACHDEM twi_write_reg zurückgekehrt ist
                 state_enter_time = current_time;
                 current_test_state = TEST_SET_HIGH_WAIT_SETTLE;
             }
             break;
        
         case TEST_SET_HIGH_WAIT_SETTLE:
             // Optional noch länger warten, falls nötig
              if (time_in_state >= 40) { // Weitere 40ms (Gesamt 50ms nach I2C-Return)
                 state_enter_time = current_time;
                 current_test_state = TEST_UART_CHECK_B_START;
             }
            break;


        case TEST_UART_CHECK_B_START:
            uart_printf("UART Check AFTER switch: Sending 'B'\r\n");
            uart_send_byte('B'); // Versuch direkt nach dem Schalten (+ kurze Wartezeit)
            state_enter_time = current_time;
            current_test_state = TEST_UART_CHECK_B_WAIT;
            break;

        case TEST_UART_CHECK_B_WAIT:
             if (time_in_state >= 50) { // Warte 50ms
                state_enter_time = current_time;
                current_test_state = TEST_I2C_CHECK_1_START;
            }
            break;

        case TEST_I2C_CHECK_1_START:
            uart_printf("I2C Check AFTER switch: Reading Config Register...\r\n");
            i2c_op_success = twi_read_reg(PCA9557_I2C_ADDR, PCA9557_REG_CONFIG, &read_value_buffer);
            if (i2c_op_success) {
                uart_printf("Read Config Reg: 0x%02X -> OK\r\n", read_value_buffer);
            } else {
                uart_printf("Read Config Reg: FAILED!\r\n");
            }
            state_enter_time = current_time;
            current_test_state = TEST_I2C_CHECK_1_WAIT;
            break;

        case TEST_I2C_CHECK_1_WAIT:
             if (time_in_state >= 500) { // Warte 500ms
                state_enter_time = current_time;
                current_test_state = TEST_RESET_HIZ_START;
            }
            break;

        case TEST_RESET_HIZ_START:
            uart_printf("Setting Pin %u back to Input (High-Z)...\r\n", test_pin);
            i2c_op_success = pca9557_set_pin_input_hiz(test_pin);
            uart_printf("Result: %s\r\n", i2c_op_success ? "OK" : "FAIL");
            state_enter_time = current_time;
            current_test_state = TEST_RESET_HIZ_WAIT_I2C;
            break;

        case TEST_RESET_HIZ_WAIT_I2C:
            if (time_in_state >= 10) { // Warte 10ms nach I2C
                state_enter_time = current_time;
                current_test_state = TEST_RESET_HIZ_WAIT_SETTLE;
            }
            break;

        case TEST_RESET_HIZ_WAIT_SETTLE:
             if (time_in_state >= 40) { // Weitere 40ms (Gesamt 50ms)
                state_enter_time = current_time;
                current_test_state = TEST_UART_CHECK_C_START;
            }
            break;

        case TEST_UART_CHECK_C_START:
            uart_printf("UART Check AFTER reset to Hi-Z: Sending 'C'\r\n");
            uart_send_byte('C');
            state_enter_time = current_time;
            current_test_state = TEST_UART_CHECK_C_WAIT;
            break;

        case TEST_UART_CHECK_C_WAIT:
             if (time_in_state >= 50) { // Warte 50ms
                state_enter_time = current_time;
                current_test_state = TEST_I2C_CHECK_2_START;
            }
            break;

        case TEST_I2C_CHECK_2_START:
            uart_printf("I2C Check AFTER reset to Hi-Z: Reading Config Register...\r\n");
            i2c_op_success = twi_read_reg(PCA9557_I2C_ADDR, PCA9557_REG_CONFIG, &read_value_buffer);
             if (i2c_op_success) {
                uart_printf("Read Config Reg: 0x%02X -> OK\r\n", read_value_buffer);
            } else {
                uart_printf("Read Config Reg: FAILED!\r\n");
            }
            state_enter_time = current_time;
            current_test_state = TEST_I2C_CHECK_2_WAIT;
            break;

        case TEST_I2C_CHECK_2_WAIT:
             if (time_in_state >= 500) { // Warte 500ms
                uart_printf("--- PCA9557 Output Switch Test Finished ---\r\n");
                state_enter_time = current_time;
                current_test_state = TEST_FINISHED; // Test beenden
            }
            break;

        case TEST_FINISHED:
            // Nichts mehr tun
            break;
    }
}
#endif // DEBUG_UART_ENABLE

// --- Main Loop Tasks ---
void main_loop_tasks(void) {
#ifdef DEBUG_UART_ENABLE
    // Führe den Test-Schritt aus, solange der Test läuft
    if (current_test_state != TEST_FINISHED) {
        run_pca_output_switch_test_step();
    } else {
        // Nach dem Test: Normales Debug-Verhalten (z.B. '.' senden)
        static uint32_t last_dot_send_time = 0;
        const uint32_t dot_interval = 1000;
        uint32_t current_time_ms = timer_get_ms();
        if ((current_time_ms - last_dot_send_time) >= dot_interval) {
            last_dot_send_time = current_time_ms;
            uart_send_byte('.');
        }
    }
#endif

    // LED Blinken (läuft immer)
    static uint32_t last_led_toggle_time = 0;
    if ((timer_get_ms() - last_led_toggle_time) >= 500) {
        last_led_toggle_time = timer_get_ms();
        PORTC.OUTTGL = PIN1_bm; // Toggle Red LED
    }
}

// --- Main Function ---
int main(void) {
    _delay_ms(20); // Kurzer Startup Delay
    initialize_system();

    // Test wird jetzt aus main_loop_tasks gestartet/durchgeführt

    while (1) {
        main_loop_tasks();
    }
    return 0;
}
