/**
 * @file main_pa3_split_fixed.c
 * @brief ATtiny826 - Generate 1kHz square wave on PA3 via TCA0 Split Mode, enable level shifter OE on PC3.
 *
 * Configures Timer/Counter A (TCA0) in Split Mode to output
 * a ~1kHz, ~50% duty cycle square wave on pin PA3 (WO3) using Single-Slope PWM.
 * Also sets PC3 (connected to Level Shifter OE) to HIGH to enable it.
 * This avoids interrupt jitter for precise measurements.
 *
 * Target MCU: ATtiny826
 * System Clock: 20 MHz (Internal OSC20M, Prescaler disabled)
 * Output Pin: PA3 (WO3)
 */

#define F_CPU 20000000UL // 20 MHz System Clock

#include <avr/io.h>
#include <avr/cpufunc.h> // For _PROTECTED_WRITE
#include <stdbool.h>

/**
 * @brief Initializes GPIO pins.
 * - PC3 (Level Shifter OE) as Output HIGH.
 * - PA3 (TCA0 WO3 Output) as Output.
 */
void PORT_init(void) {
    // Configure PC3 (OE for Level Shifter) as output and set it HIGH
    PORTC.DIRSET = PIN3_bm; // PC3 direction = Output
    PORTC.OUTSET = PIN3_bm; // PC3 output level = High

    // Configure PA3 (TCA0 Waveform Output 3) as output
    // TCA0 peripheral override will control the pin when TCA0 is enabled.
    PORTA.DIRSET = PIN3_bm; // PA3 direction = Output
}

/**
 * @brief Initializes Timer/Counter A (TCA0) in Split Mode for ~1kHz/~50% PWM on WO3 (PA3).
 * - Uses High Byte counter (HCNT) for WO3.
 * - Implicitly uses Single-Slope PWM (only mode in Split).
 * - Uses CLK_PER (20 MHz) as source, prescaled by 256.
 * - Sets HPER and HCMP0 for target frequency and duty cycle.
 */
void TCA0_init_split_pa3(void) {
    // Calculations remain the same
    uint8_t high_period_value = 77; // HPER
    uint8_t high_compare0_value = 39; // HCMP0 for WO3

    // --- Configure TCA0 ---

    // 1. Ensure TCA0 is disabled
    TCA0.SINGLE.CTRLA &= ~TCA_SINGLE_ENABLE_bm;

    // Optional: Reset TCA0
    TCA0.SINGLE.CTRLESET = TCA_SINGLE_CMD_RESET_gc;
    while (TCA0.SINGLE.CNT != 0) {;}

    // 2. Enable Split Mode (!! Must be done BEFORE accessing SPLIT registers !!)
    TCA0.SINGLE.CTRLD = TCA_SPLIT_SPLITM_bm;

    // 3. Configure High Byte Timer (controls WO3, WO4, WO5)
    TCA0.SPLIT.HPER = high_period_value;
    TCA0.SPLIT.HCMP0 = high_compare0_value;
    TCA0.SPLIT.HCMP1 = 0;
    TCA0.SPLIT.HCMP2 = 0;
    TCA0.SPLIT.HCNT = 0;

    // 4. Configure Low Byte Timer (controls WO0, WO1, WO2) - Defaults
    TCA0.SPLIT.LPER = 0xFF;
    TCA0.SPLIT.LCMP0 = 0;
    TCA0.SPLIT.LCMP1 = 0;
    TCA0.SPLIT.LCMP2 = 0;
    TCA0.SPLIT.LCNT = 0;

    // 5. Configure the single CTRLB register for Split Mode
    // Enable High Byte Compare Output 0 (HCMP0EN) for WO3 (PA3).
    // Ensure Low Byte Compare Outputs (LCMPnEN) are disabled (cleared).
    // The reset value is 0x00, so we only need to set the desired bit.
    TCA0.SPLIT.CTRLB = TCA_SPLIT_HCMP0EN_bm; // Use the single CTRLB register

    // 6. Configure Global TCA0 settings (Clock source applies to both halves)
    TCA0.SPLIT.CTRLA = TCA_SPLIT_CLKSEL_DIV256_gc; // Prescaler DIV256

    // 7. Enable TCA0 globally (applies to split mode)
    TCA0.SPLIT.CTRLA |= TCA_SPLIT_ENABLE_bm;
}


/**
 * @brief Initializes Timer/Counter A (TCA0) in Split Mode for ~1kHz/~50% PWM on WO4 (PA4).
 *        This is an alternative if PA3 is not suitable.
 */
void TCA0_init_split_pa4(void) {
    // Calculations are the same as for PA3, but use HCMP1
    uint8_t high_period_value = 77; // HPER
    uint8_t high_compare1_value = 39; // HCMP1 for WO4

    // --- Configure TCA0 ---
    TCA0.SINGLE.CTRLA &= ~TCA_SINGLE_ENABLE_bm; // Disable first
    TCA0.SINGLE.CTRLESET = TCA_SINGLE_CMD_RESET_gc; // Reset
    while (TCA0.SINGLE.CNT != 0) {;}

    TCA0.SINGLE.CTRLD = TCA_SPLIT_SPLITM_bm; // Enable Split Mode

    // Configure High Byte Timer
    TCA0.SPLIT.HPER = high_period_value;
    TCA0.SPLIT.HCMP0 = 0;
    TCA0.SPLIT.HCMP1 = high_compare1_value; // Set compare value for WO4 (PA4)
    TCA0.SPLIT.HCMP2 = 0;
    TCA0.SPLIT.HCNT = 0;

    // Configure Low Byte Timer (defaults)
    TCA0.SPLIT.LPER = 0xFF;
    TCA0.SPLIT.LCMP0 = 0; TCA0.SPLIT.LCMP1 = 0; TCA0.SPLIT.LCMP2 = 0;
    TCA0.SPLIT.LCNT = 0;

    // Configure the single CTRLB register for Split Mode
    // Enable High Byte Compare Output 1 (HCMP1EN) for WO4 (PA4).
    // Ensure Low Byte and other High Byte Compare Outputs are disabled.
    TCA0.SPLIT.CTRLB = TCA_SPLIT_HCMP1EN_bm; // Use the single CTRLB register

    // Configure Global TCA0 settings
    TCA0.SPLIT.CTRLA = TCA_SPLIT_CLKSEL_DIV256_gc; // Prescaler DIV256

    // Enable TCA0 globally
    TCA0.SPLIT.CTRLA |= TCA_SPLIT_ENABLE_bm;
}


/**
 * @brief Main application entry point.
 */
int main(void) {
    // --- System Clock Configuration ---
    _PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0); // No division, PEN = 0

    // --- Peripheral Initialization ---
    PORT_init();

    // --- Choose ONE ---
    TCA0_init_split_pa3();  // Output on PA3 (WO3)
    // TCA0_init_split_pa4(); // Alternative: Output on PA4 (WO4)

    // --- Main Loop ---
    while (1) {
        // Timer runs independently
    }
}
